/**
 *	Copyright (c) 2008,  Eckhart Arnold
 *
 * Dieses Programm ist freie Software. Sie können es unter den
 * Bedingungen der GNU General Public License, wie von der Free
 * Software Foundation veröffentlicht, weitergeben und/oder
 * modifizieren, entweder gemäß Version 3 der Lizenz oder (nach
 * Ihrer Option) jeder späteren Version.
 *
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung,
 * daß es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE
 * GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder
 * der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden
 * Sie in der GNU General Public License.
 *
 * Sie sollten ein Exemplar der GNU General Public License zusammen
 * mit diesem Programm erhalten haben. Falls nicht, siehe
 * <http://www.gnu.org/licenses/>.
 */

package evolution.examples;

import java.util.ArrayList;

public class TailGenome implements IGenome {
	private TSPData data;
	private int[] tail, path;
	double stretch = -1.0;
	
	TailGenome(TSPData data) {
		this.data = data;
		tail = new int[data.points];
		path = new int[data.points];
		for (int i = 0; i < tail.length; i++) {
			tail[i] = EAUtil.rnd.nextInt(tail.length - i);
		}
	}

	TailGenome(TailGenome copy) {
		this.data = copy.data;
		this.tail = copy.tail; // .clone();
		this.path = copy.path; // .clone();
		this.stretch = copy.stretch;
	}
	@Override
	public synchronized IGenome clone() {
		return new TailGenome(this);
	}
	
	public double roundTrip() {
		double x1, y1, x2, y2, dx, dy;
		if (stretch >= 0.0) return stretch;
		getPath();
		stretch = 0.0;
		x1 = data.location[path[path.length-1]][0];
		y1 = data.location[path[path.length-1]][1];
		for (int i = 0; i < path.length; i++) {
			x2 = data.location[path[i]][0];
			y2 = data.location[path[i]][1];
			dx = x2-x1;  
			dy = y2-y1;
			stretch += Math.sqrt(dx*dx + dy*dy);
			x1 = x2;  
			y1 = y2;
		}
		return stretch;
	}
	
	
	public void mutate(int a) {
		if (stretch >= 0.0) tail = tail.clone(); 
		tail[a] = EAUtil.rnd.nextInt(tail.length-a);
		stretch = -1.0;
	}
	
	public void exchange(IGenome other, int from, int to) {
		if (stretch >= 0.0) tail = tail.clone();  // copy on write 
		if (((TailGenome) other).stretch >= 0.0) 
			((TailGenome) other).tail = ((TailGenome) other).tail.clone(); 
		
		// two point crossover
		for (int i = from; i < to; i++) {
			int swap = tail[i];
			tail[i] = ((TailGenome) other).tail[i];
			((TailGenome) other).tail[i] = swap;
		}
		stretch = -1.0;			// invalidate
		((TailGenome) other).stretch = -1.0;
	}	
	

	
	public void repair() {
		// no repairs necessary !
	}
	
	public int[] getPath() {
		if (stretch >= 0.0) return path;
		path = new int[tail.length]; // copy on write
		ArrayList<Integer> points = new ArrayList<Integer>(tail.length);
		for (int i = 0; i < tail.length; i++) points.add(i, i);
		for (int i = 0; i < tail.length; i++) path[i] = points.remove(tail[i]);
		return path;	
	}
	
	public int getLength() { return tail.length; }
}
