/**
 *	Copyright (c) 2008,  Eckhart Arnold
 *
 * Dieses Programm ist freie Software. Sie können es unter den
 * Bedingungen der GNU General Public License, wie von der Free
 * Software Foundation veröffentlicht, weitergeben und/oder
 * modifizieren, entweder gemäß Version 3 der Lizenz oder (nach
 * Ihrer Option) jeder späteren Version.
 *
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung,
 * daß es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE
 * GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder
 * der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden
 * Sie in der GNU General Public License.
 *
 * Sie sollten ein Exemplar der GNU General Public License zusammen
 * mit diesem Programm erhalten haben. Falls nicht, siehe
 * <http://www.gnu.org/licenses/>.
 */

package evolution.examples;

import java.util.ArrayList;
import java.util.Random;

public class EAUtil {
	public static Random rnd = new Random(5000);
	
	public class RandomIndices {
		private ArrayList<Integer> pickList;
		
		RandomIndices(int size) {
			pickList = new ArrayList<Integer>(size);
			for (int i = 0; i < size; i++) 
				pickList.add(i, i);
		}	
		public int next() {
			return pickList.remove(rnd.nextInt(pickList.size()));
		}
		public int left() { return pickList.size(); }
	}
	
	public class Roulette {
		private double wheel[];
		public double pointer, step;
		
		Roulette(double[] distribution) {
			wheel = new double[distribution.length];
			wheel[0] = distribution[0];
			for (int i = 1; i < distribution.length; i++) 
				wheel[i] = wheel[i-1] + distribution[i];
			pointer = rnd.nextDouble() / distribution.length;
			step = 1.0 / distribution.length;
		}		
		private int select(double r) {
			assert r < 1.0;
			int i = 0, k = wheel.length-1, m=0;
			while (i < k) {
				m = (i+k)/2;
				if (r > wheel[m]) i = m+1;
				else k = m;
			}
			return i;			
		}
		public int pick() {
			double r = rnd.nextDouble();
			return select(r);
		}
		public boolean hasNext() {
			return pointer < 1.0;
		}
		public int next() {
			assert hasNext();
			
			int m = select(pointer);
			pointer += step;
			return m;
		}
	}
}
