/**
 *	Copyright (c) 2008,  Eckhart Arnold
 *
 * Dieses Programm ist freie Software. Sie können es unter den
 * Bedingungen der GNU General Public License, wie von der Free
 * Software Foundation veröffentlicht, weitergeben und/oder
 * modifizieren, entweder gemäß Version 3 der Lizenz oder (nach
 * Ihrer Option) jeder späteren Version.
 *
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung,
 * daß es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE
 * GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder
 * der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden
 * Sie in der GNU General Public License.
 *
 * Sie sollten ein Exemplar der GNU General Public License zusammen
 * mit diesem Programm erhalten haben. Falls nicht, siehe
 * <http://www.gnu.org/licenses/>.
 */

package evolution.examples;

import org.eclipse.swt.graphics.*;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * @author eckhart
 *
 */
public class Monitor {
	protected class Graph {
		int n = 0;
		ArrayList<Double> vl = new ArrayList<Double>();
		int rgb = 0x000000;
	}
	
	protected HashMap<String, Graph> values;
	protected HashMap<String, Integer> tracker;
	protected int maxX = 10;
	protected double minY = 0.0, maxY = 1.0;
	protected double ymult, xmult, dy, xstretch;
	protected boolean rescale = true, disposed = false;
	
	public Monitor(String[] pens) {
		values = new HashMap<String, Graph>();
		tracker = new HashMap<String, Integer>();
		int[] rgbValues = { 0x00FF00, 0xFF0000, 0x0000FF, 0x000000 };
		int i = 0;
		for (String p : pens) {
			Graph g = new Graph();
			g.rgb = rgbValues[i++ % rgbValues.length];
			values.put(p, g);
			tracker.put(p, 0);
		}
	}
	
	int trX(int x) {
		return (int) (x * xmult + 0.5);
	}
	
	int trY(double y) {
		return (int) (y * ymult + dy + 0.5);
	}
	
	void redraw(GC gc) {
		Rectangle r = gc.getClipping();
		Color oldC, color = new Color(gc.getDevice(), 255, 255, 255);
		oldC = gc.getBackground();
		gc.setBackground(color);
		gc.fillRectangle(r);
		gc.setBackground(oldC);
		color.dispose();
		ymult = -r.height/(maxY-minY);
		xmult = r.width/((double) maxX);
		dy = r.height-(minY * ymult);
		xstretch = Math.log10(maxX);
		rescale = false;
		for (String pen: tracker.keySet()) tracker.put(pen, 0);
		update(gc);
	}

	void update(GC gc) {
		if (rescale) redraw(gc);
		else {
			Graph graph;
			Color color, oldC;
			int pos, len;
			for (String pen: values.keySet()) {
				graph = values.get(pen);
				int r = graph.rgb >> 16 & 0xFF;
				int g = graph.rgb >> 8 & 0xFF;
				int b = graph.rgb & 0xFF;
				pos = tracker.get(pen);
				len = graph.vl.size() - pos;
				int[] points = new int[len*2];
				for (int i = 0; i < len; i++) {
					points[i*2] = trX(invIndex(i + pos));
					points[i*2+1] = trY(graph.vl.get(i+pos));
				}
				color = new Color(gc.getDevice(), r, g, b);
				oldC = gc.getForeground();
				gc.setForeground(color);
				gc.drawPolyline(points);
				gc.setForeground(oldC);
				color.dispose();
				tracker.put(pen, Math.max(pos+len-1, 0));
			}
		}
	}
	
	int index(int x) {
		// return (new Double(Math.log(((double)(x+100)) / 100.0 )*100.0)).intValue();
		return (int) (Math.log(((double)(x+100)) / 100.0 )*100.0);
	}
	
	int invIndex(int i) {
		return (int) (Math.exp(((double)i) / 100.0) * 100.0 - 100.0); 
	}	
	
	void addValue(String pen, double y) {
		Graph graph = values.get(pen);
		graph.n++;
		if (graph.vl.size() <= index(graph.n)) {
			graph.vl.add(y); 
			if (y < minY) { minY = y; rescale = true; }
			if (y > maxY) { maxY = y; rescale = true; }
		}
		if (graph.n > maxX) {
			maxX = 2*maxX; 
			rescale = true; 
		}		
	}
}

